@@ -40,7 +40,9 @@ module Test.Hspec.Expectations (
 import           Test.HUnit (Assertion, (@?=), assertBool, assertFailure)
 import           Control.Exception
 import           Data.Typeable
-import           Data.List ((\\), isInfixOf)
+import           Data.List (isInfixOf)
+
+import           Test.Hspec.Expectations.Matcher
 
 type Expectation = Assertion
 
@@ -63,7 +65,7 @@ v `shouldSatisfy` p = assertBool ("predicate failed on: " ++ show v) (p v)
 
 -- |
 -- @list \`shouldContain\` sublist@ sets the expectation that @sublist@ is contained,
--- wholly and intact, anywhere in the second.
+-- wholly and intact, anywhere in @list@.
 shouldContain :: (Show a, Eq a) => [a] -> [a] -> Expectation
 list `shouldContain` sublist = assertBool errorMsg (sublist `isInfixOf` list)
   where
@@ -73,9 +75,7 @@ list `shouldContain` sublist = assertBool errorMsg (sublist `isInfixOf` list)
 -- @xs \`shouldMatchList\` ys@ sets the expectation that @xs@ has the same
 -- elements that @ys@ has, possibly in another order
 shouldMatchList :: (Show a, Eq a) => [a] -> [a] -> Expectation
-xs `shouldMatchList` ys = assertBool errorMsg (all null [xs \\ ys, ys \\ xs])
-  where
-    errorMsg = show ys ++ " is not a permutation of " ++ show xs
+xs `shouldMatchList` ys = maybe (return ()) assertFailure (matchList xs ys)
 
 -- |
 -- @action \`shouldReturn\` expected@ sets the expectation that @action@
